@@ -10,6 +10,7 @@ from typing import Callable
 from typing import Dict
 from typing import Iterable
 from typing import List
+from typing import Set
 from typing import Tuple
 from typing import Union
 
@@ -20,6 +21,7 @@ from falocalrepo_server import __version__ as __server_version__
 from falocalrepo_server import server
 
 from .__version__ import __version__
+from .commands import check_database
 from .commands import check_process
 from .commands import latest_version
 from .commands import make_journal
@@ -148,6 +150,7 @@ def help_(comm: str = "", op: str = "", *_rest) -> str:
         "database server": database_server,
         "database merge": database_merge,
         "database clean": database_clean,
+        "database upgrade": database_upgrade,
     }.get(comm := f"{comm} {op}".strip(), None)
 
     if f is None:
@@ -156,7 +159,7 @@ def help_(comm: str = "", op: str = "", *_rest) -> str:
     return cleandoc(f.__doc__)
 
 
-def init():
+def init(db: FADatabase):
     """
     USAGE
         falocalrepo init
@@ -169,6 +172,7 @@ def init():
         update it to a new version without calling other commands.
     """
 
+    check_database(db.version, __database_version__)
     print("Database ready")
 
 
@@ -253,6 +257,8 @@ def config(db: FADatabase, comm: str = "", *args: str):
         The config command allows to change the settings used by the program.
     """
 
+    check_database(db.version, __database_version__)
+
     {
         "": config_list,
         "list": config_list,
@@ -406,10 +412,10 @@ def download(db: FADatabase, comm: str = "", *args: str):
         thumbnails, if there are any.
     """
 
-    if not comm:
-        raise MalformedCommand("download needs a command")
+    check_database(db.version, __database_version__)
 
     {
+        "": raiser(MalformedCommand("download needs a command")),
         "users": download_users,
         "update": download_update,
         "submissions": download_submissions,
@@ -430,8 +436,8 @@ def database_info(db: FADatabase, *_rest):
     print("Users       :", len(db.users))
     print("Submissions :", len(db.submissions))
     print("Journals    :", len(db.journals))
-    print("History     :", len(db.settings.read_history()) - 1)
-    print("Version     :", db.settings["VERSION"])
+    print("History     :", (len(h) - 1) if (h := db.settings.read_history()) else 0)
+    print("Version     :", db.version)
 
 
 def database_history(db: FADatabase):
@@ -550,13 +556,13 @@ def database_add_user(db: FADatabase, *args):
     """
 
     make_params: Dict[str, str] = parameters(args)
-    db.users.new_user(username) if (username := make_params["username"]) not in db.users else None
+    username: str = db.users.new_user(make_params["username"])
     if make_params.get("folders", None) is not None:
-        folders: List[str] = db.users[username]["FOLDERS"].split(",")
-        folders_new: List[str] = make_params["folders"].split(",")
-        for f in folders:
+        folders: Set[str] = set(db.users[username]["FOLDERS"])
+        folders_new: Set[str] = set(filter(bool, map(str.lower, make_params["folders"].split(","))))
+        for f in folders - folders_new:
             db.users.remove_user_folder(username, f) if f not in folders_new else None
-        for f in folders_new:
+        for f in folders_new - folders:
             db.users.add_user_folder(username, f) if f not in folders else None
     db.commit()
 
@@ -759,6 +765,19 @@ def database_clean(db: FADatabase, *_rest):
     db.vacuum()
 
 
+@docstring_parameter(__database_version__)
+def database_upgrade(db: FADatabase, *_rest):
+    """
+    USAGE
+        falocalrepo database upgrade
+
+    DESCRIPTION
+        Upgrade the database to the latest version ({0}).
+    """
+
+    db.upgrade()
+
+
 @docstring_parameter(f"https://pypi.org/project/falocalrepo-database/{__database_version__}")
 def database(db: FADatabase, comm: str = "", *args: str):
     """
@@ -785,6 +804,7 @@ def database(db: FADatabase, comm: str = "", *args: str):
         server              Start local server to browse database
         merge               Merge with a second database
         clean               Clean the database with the VACUUM function
+        upgrade             Upgrade the database to the latest version.
 
     DESCRIPTION
         The database command allows to operate on the database. Used without an
@@ -806,6 +826,9 @@ def database(db: FADatabase, comm: str = "", *args: str):
         specific search command.
     """
 
+    if comm not in ("", "info", "upgrade"):
+        check_database(db.version, __database_version__)
+
     {
         "": database_info,
         "info": database_info,
@@ -822,6 +845,7 @@ def database(db: FADatabase, comm: str = "", *args: str):
         "server": database_server,
         "merge": database_merge,
         "clean": database_clean,
+        "upgrade": database_upgrade,
     }.get(comm, raiser(UnknownCommand(f"database {comm}")))(db, *args)
 
 
@@ -901,12 +925,11 @@ def console(comm: str = "", *args: str) -> None:
     db: FADatabase = FADatabase(abspath(database_path))
 
     try:
-        db.upgrade()
         db.settings.add_history(f"{comm} {' '.join(args)}".strip())
         db.commit()
 
         if comm == init.__name__:
-            init()
+            init(db)
         elif comm == config.__name__:
             config(db, *args)
         elif comm == download.__name__:
@@ -914,7 +937,6 @@ def console(comm: str = "", *args: str) -> None:
         elif comm == database.__name__:
             database(db, *args)
     finally:
-        # Close database and update totals
         if db is not None:
             db.commit()
             db.close()
